﻿Imports System.IO.Ports
Imports System.Threading

''' <summary>
''' ＤＩＰＰＬＣと通信するためのクラス
''' </summary>
Public Class DipPlc
    Inherits IO.Ports.SerialPort
    '    Private serial As IO.Ports.SerialPort
    ''' <summary>
    ''' デバイスの名前を表す定数群　
    ''' </summary>
    Enum DevName
        X = 0
        Y = 1
        M = 2
        T = 3
        C = 4
        D = 10
    End Enum
    Enum ADIO_MODE
        AD4 = 0
        AD5 = 1
        DI4 = 2
        DI5 = 3
    End Enum

    Public Class CMD
        Public Const EXE_CMD As String = "R" ' 実行コマンド（プログラムを実行する）
        Public Const EXE_PAUSE As Byte = &H3  ' 中断コマンド（ラダーを中断しプロンプトを表示、コマンド待ち）
        Public Const EXE_RESET As String = "@"  ' リセットコマンド
        Public Const EXE_VIEW As String = "V"   ' システム情報表示コマンド

        Public Const SET_AD4 As String = "X04A"
        Public Const SET_AD5 As String = "X05A"
        Public Const SET_DI4 As String = "X04D"
        Public Const SET_DI5 As String = "X05D"

        Public Const GET_LIST As String = "L"         ' メモリー上のラダーニーモニックを表示する
        Public Const GET_LIST_NUM As String = "L"      ' 指定した行以降を表示する
        Public Const PG_SET As String = ""
        Public Const PG_INS As String = "I"
        Public Const ERR_CODE As String = "???"

        Public Const ERACE As String = "N"
        Public Const ERACE_ALL As String = "NA"
    End Class

    ''' <summary>
    ''' デバイスのＥｎｕｍから文字を取得するための配列
    ''' </summary>
    Private DevNameStr() As String = New String() {"X", "Y", "M", "T", "C", "", "", "", "", "", "D"}

    ''' <summary>
    ''' 同期処理時の送信と受信の間の待ち時間
    ''' </summary>
    Private WaitTime As Integer

    ''' <summary>
    ''' 非同期通信で受信したデータを格納する変数
    ''' </summary>
    Private recvStr As String
    ''' <summary>
    ''' 非同期通信時に使用するミューテックス
    ''' </summary>
    Private mtx As New Mutex
    ''' <summary>
    ''' 非同期通信のイベントを登録したかどうかを判定するフラグ変数
    ''' </summary>
    Private handleFlag As Boolean

    Public Sub New()
        '        serial = New IO.Ports.SerialPort()
        MyBase.BaudRate = 9600
        MyBase.DataBits = 8
        MyBase.StopBits = IO.Ports.StopBits.One
        MyBase.Parity = IO.Ports.Parity.None
        MyBase.Handshake = IO.Ports.Handshake.XOnXOff
        MyBase.ReadTimeout = 50
        MyBase.Encoding = New System.Text.ASCIIEncoding()
        MyBase.NewLine = vbCrLf

        WaitTime = 50
        handleFlag = False
    End Sub

    ''' <summary>
    ''' DIPPLCを検索し、通信できるDIPPLCのポート番号をListにして返す
    '''    Throw ArgumentException 開始番号より終了番号が小さいときに発生
    ''' </summary>
    ''' <param name="startNum">調査開始ポート番号</param>
    ''' <param name="endNum">最終ポート番号</param>
    ''' <returns>見つかったDIPPLCのポート名をList(of String)にして返す</returns>
    Public Function DipPlcSearch(ByVal startNum As Integer, ByVal endNum As Integer) As List(Of String)
        If IsOpen Then
            Throw New Exception("ポートが開いています。閉じてから行ってください。")
        End If

        Dim pstrList As List(Of String) = New List(Of String)
        Dim number As Integer

        If (startNum > endNum) Then
            Throw New ArgumentException("開始番号は終了番号より小さくしてください。")
        End If

        For number = startNum To endNum
            If DipPlcCheck(number) = True Then
                pstrList.Add("COM" & number)
            Else

            End If
        Next

        Return pstrList
    End Function
    ''' <summary>
    ''' 通信ができるCOMポートを検索し、通信できるポート番号をListにして返す
    '''     Throw ArgumentException 開始番号より終了番号が小さいときに発生 
    ''' </summary>
    ''' <param name="startNum">調査開始ポート番号</param>
    ''' <param name="endNum">最終ポート番号</param>
    ''' <returns>見つかったCOMのポート名をList(of String)にして返す</returns>
    Public Function PortSearch(ByVal startNum As Integer, ByVal endNum As Integer) As List(Of String)
        If IsOpen Then
            Throw New Exception("ポートが開いています。閉じてから行ってください。")
        End If

        Dim pstrList As List(Of String) = New List(Of String)
        Dim number As Integer

        If (startNum > endNum) Then
            Throw New ArgumentException("開始番号は終了番号より小さくしてください。")
        End If

        For number = startNum To endNum
            If PortCheck(number) = True Then
                pstrList.Add("COM" & number)
            Else

            End If
        Next

        Return pstrList
    End Function
    ''' <summary>
    ''' DIPPLCかどうかをチェックする関数。DIPPLCと通信できた True, 通信できなかった Falseを返す。
    '''   Throw  ArgumentOutOfRangeException("ポート番号が０以下です！")
    ''' </summary>
    ''' <param name="number">通信を仕掛けるポート番号</param>
    ''' <returns>DIPPLCと通信できた True, 通信できなかった False</returns>
    Public Function DipPlcCheck(ByVal number As Integer) As Boolean
        If IsOpen Then
            Throw New Exception("ポートが開いています。閉じてから行ってください。")
            Return False
        End If

        Dim Cmd As String = ""
        Dim Recv As String = ""
        Dim Flag As Boolean = False
        Dim bkReceive As Integer = ReceivedBytesThreshold

        If (number <= 0) Then
            Throw New ArgumentOutOfRangeException("ポート番号が０以下です！")
        End If

        MyBase.PortName = "COM" & number
        MyBase.ReceivedBytesThreshold = 100

        Try
            MyBase.Open() ' 開いてみる

            MyBase.ReadExisting() ' 入力バッファをとりあえず読み出す
            Flag = True
        Catch ex As UnauthorizedAccessException ' すでにほかのアプリケーションで使用しています。
            Flag = False
        Catch ex As ArgumentOutOfRangeException ' 通信設定が間違っています
            Flag = False
        Catch ex As ArgumentException   '   ' ポート名が”COMで始まっていない”
            Flag = False
        Catch ex As InvalidOperationException   ' 同じアプリケーション内ですでに開いています
            Flag = False
        Catch ex As IO.IOException      ' ポートが存在しない
            Flag = False
        End Try

        If Flag = False Then
            MyBase.ReceivedBytesThreshold = bkReceive
            Return False
        End If

        Try
            Cmd = "X00?"

            MyBase.WriteLine(Cmd)
            Threading.Thread.Sleep(50)
            Do
                Recv &= MyBase.ReadLine() & vbLf
            Loop
        Catch ex As TimeoutException
            If (Recv.IndexOf("X00:") >= 0) Then
                Flag = True
            Else
                Flag = False
            End If
        Finally
            Close()
            MyBase.ReceivedBytesThreshold = bkReceive
        End Try
        Return Flag
    End Function
    ''' <summary>
    ''' ポートが使用できるかどうかをチェックする関数。ポートが開けた True, 開けなかった Falseを返す。
    '''   Throw  ArgumentOutOfRangeException("ポート番号が０以下です！")
    ''' </summary>
    ''' <param name="number">通信を仕掛けるポート番号</param>
    ''' <returns>ポートが開けた True, 通信できなかった False</returns>
    Public Function PortCheck(ByVal number As Integer) As Boolean
        If IsOpen Then
            Throw New Exception("ポートが開いています。閉じてから行ってください。")
            Return False
        End If

        If (number <= 0) Then
            Throw New ArgumentOutOfRangeException("ポート番号が０以下です！")
        End If

        MyBase.PortName = "COM" & number

        Try
            MyBase.Open() ' 開いてみる

            Close() ' 正常終了したとき
            Return True
        Catch ex As UnauthorizedAccessException ' すでにほかのアプリケーションで使用しています。
            Return False
        Catch ex As ArgumentOutOfRangeException ' 通信設定が間違っています
            Return False
        Catch ex As ArgumentException   '   ' ポート名が”COMで始まっていない”
            Return False
        Catch ex As InvalidOperationException   ' 同じアプリケーション内ですでに開いています
            Return False
        Catch ex As IO.IOException      ' ポートが存在しない
            Return False
        End Try
    End Function
    ''' <summary>
    ''' 通信のプロトコルを独自設定するメソッド
    ''' </summary>
    ''' <param name="portname">ポート番号</param>
    ''' <param name="databits">データ長</param>
    ''' <param name="stopbits">ストップビット</param>
    ''' <param name="parity">パリティビット</param>
    ''' <param name="hand">ハンドシェイク</param>
    Public Sub SetMode(ByRef portname As String, ByVal databits As Integer, ByVal stopbits As IO.Ports.StopBits, ByVal parity As IO.Ports.Parity, ByVal hand As IO.Ports.Handshake)
        If (IsOpen = False) Then
            MyBase.PortName = portname
            MyBase.DataBits = databits
            MyBase.StopBits = stopbits
            MyBase.Parity = parity
            MyBase.Handshake = hand
        End If
    End Sub
    ''' <summary>
    ''' 通信ポートを閉じるメソッド。内部でSerialPortクラスのCloseを呼び出している。
    ''' </summary>
    Overloads Sub Close()
        If (MyBase.IsOpen) Then
            MyBase.Close()
        End If
    End Sub

    ''' <summary>
    ''' １つのビットデバイスのＯＮ／ＯＦＦ状態を取得するメソッド。
    ''' </summary>
    ''' <param name="dev">DevName列挙型のX,Y,T,M,Cを指定できる</param>
    ''' <param name="number">デバイスの番号</param>
    ''' <returns>True:ＯＮ　False:ＯＦＦ</returns>
    Function GetBitDevice(ByVal dev As DevName, ByVal number As Integer) As Boolean
        If Not (IsOpen) Then
            Throw New Exception("ポートが開かれていません。")
            Return False
        End If

        Dim Cmd As String
        Dim DevStr As String
        Dim DevNum As String
        Dim RecvData As String = ""

        If (dev >= DevName.X And dev <= DevName.C) Then
            DevStr = DevNameStr(dev)
        Else
            Throw New Exception("使用できないデバイスが指定されました。")
            Return False
        End If
        Select Case number
            Case DevName.X
                If Not (number >= 0 And number <= 7) Then
                    Throw New Exception("使用できないデバイス番号が指定されました。")
                End If
            Case DevName.Y
                If Not (number >= 0 And number <= 5) Then
                    Throw New Exception("使用できないデバイス番号が指定されました。")
                End If
            Case DevName.T
                If Not (number >= 0 And number <= 15) Then
                    Throw New Exception("使用できないデバイス番号が指定されました。")
                End If
            Case DevName.C
                If Not (number >= 0 And number <= 7) Then
                    Throw New Exception("使用できないデバイス番号が指定されました。")
                End If
            Case DevName.M
                If Not (number >= 0 And number <= 63) Then
                    Throw New Exception("使用できないデバイス番号が指定されました。")
                End If
            Case Else
                Throw New Exception("使用できないデバイスが指定されました。")
        End Select
        DevNum = "0" & Convert.ToString(number)
        DevNum = DevNum.Substring(DevNum.Length - 2)

        Cmd = DevStr & DevNum & "?"

        RecvData = QueryCmd(Cmd)

        Cmd = Cmd.Replace("?", ":")

        Dim pos As Integer
        pos = RecvData.IndexOf(Cmd)

        If pos >= 0 Then
            Dim stat As String = RecvData.Substring(pos + Cmd.Length, 1)

            Select Case stat
                Case "0"
                    Return False
                Case "1"
                    Return True
                Case Else
                    Throw New Exception("応答が間違っています。")
            End Select
        Else
            Throw New Exception("正しく応答されませんでした。")
        End If
    End Function
    ''' <summary>
    ''' １つのワードデバイスの値を問い合わせてその結果を返すメソッド
    ''' </summary>
    ''' <param name="dev">DevName列挙型のDを指定できる</param>
    ''' <param name="number">デバイスの番号</param>
    ''' <returns>デバイスの値-32768 ～ 32767</returns>
    Function GetWordDevice(ByVal dev As DevName, ByVal number As Integer) As Integer
        If Not (IsOpen) Then
            Throw New Exception("ポートが開かれていません。")
            Return False
        End If

        Dim Cmd As String
        Dim DevStr As String
        Dim DevNum As String
        Dim RecvData As String = ""

        If dev = DevName.D Then
            DevStr = DevNameStr(dev)
        Else
            Throw New Exception("使用できないデバイスが指定されました。")
        End If

        If Not (number >= 0 And number <= 7) Then
            Throw New Exception("使用できないデバイス番号が指定されました。")
        End If

        DevNum = "0" & Convert.ToString(number)
        DevNum = DevNum.Substring(DevNum.Length - 2)

        Cmd = DevStr & DevNum & "?"

        RecvData = QueryCmd(Cmd)

        Cmd = Cmd.Replace("?", ":")

        Dim pos As Integer
        pos = RecvData.IndexOf(Cmd)

        If pos >= 0 Then
            Dim stat As String = RecvData.Substring(pos + Cmd.Length, 4)
            Dim data As Int16
            If Int16.TryParse(stat, System.Globalization.NumberStyles.HexNumber, Nothing, data) = False Then
                Throw New Exception("正しく数値に変換できませんでした。")
            Else
                Return data
            End If
        Else
            Throw New Exception("正しく応答されませんでした。")
        End If
    End Function
    ''' <summary>
    ''' 指定したワードデバイスのすべてを一括して問合せし、取得した結果をListにして返す。デバイス番号と要素番号は対応。
    ''' </summary>
    ''' <param name="dev">DevName列挙型のDを指定できる</param>
    ''' <returns>ワードデバイスをList(of Integer)にして返す</returns>
    Function GetWordDeviceAll(ByVal dev As DevName) As List(Of Integer)
        If Not (IsOpen) Then
            Throw New Exception("ポートが開かれていません。")
        End If

        Dim Cmd As String
        Dim DevStr As String
        Dim RecvData As String = ""

        If dev = DevName.D Then
            DevStr = DevNameStr(dev)
        Else
            Throw New Exception("使用できないデバイスが指定されました。")
        End If

        Cmd = DevStr & "?"

        Dim temptime As Integer
        temptime = WaitTime
        WaitTime = 1000
        RecvData = QueryCmd(Cmd)
        WaitTime = temptime

        Cmd = Cmd.Replace("?", ":")

        Dim pos As Integer
        pos = RecvData.IndexOf(Cmd)

        Dim aData As List(Of Integer)

        If pos >= 0 Then
            Dim stat As String = RecvData.Substring(pos + Cmd.Length())
            Dim WordStr As String
            Dim i As Integer
            Dim startpos As Integer
            Dim Data As Integer
            aData = New List(Of Integer)
            For i = 0 To 7
                WordStr = stat.Substring(0, 4)
                If Int16.TryParse(WordStr, System.Globalization.NumberStyles.HexNumber, Nothing, Data) = False Then
                    Throw New Exception("正しく数値に変換できませんでした。")
                Else
                    '                Return data
                    aData.Add(Data)
                End If
                startpos = stat.IndexOf(")")
                If startpos >= 0 Then
                    stat = stat.Substring(startpos + 2)
                Else
                    Throw New Exception("「)」が見つかりませんでした。")
                End If
            Next
        Else
            Throw New Exception("正しく応答されませんでした。")
        End If
        Return aData
    End Function
    ''' <summary>
    ''' 指定したビットデバイスのすべてを一括して問合せし、取得した結果をListにして返す。デバイス番号と要素番号は対応。
    ''' </summary>
    ''' <param name="dev">DevName列挙型のX,Y,T,M,Cを指定できる</param>
    ''' <returns>ビットデバイスをList(of Integer)にして返す</returns>
    Public Function GetBitDeviceAll(ByVal dev As DevName) As List(Of Boolean)
        If Not (IsOpen) Then
            Throw New Exception("ポートが開かれていません。")
        End If

        Dim Cmd As String
        Dim DevStr As String
        Dim RecvData As String = ""
        Dim tempTime As Integer

        If (dev >= DevName.X And dev <= DevName.C) Then
            DevStr = DevNameStr(dev)
        Else
            Throw New Exception("使用できないデバイスが指定されました。")
            Return Nothing
        End If


        Cmd = DevStr & "?"

        If dev = DevName.M Then
            tempTime = WaitTime
            WaitTime = 1000
        End If

        RecvData = QueryCmd(Cmd)

        If dev = DevName.M Then
            WaitTime = tempTime
        End If

        Cmd = Cmd.Replace("?", ":")

        Dim pos As Integer
        pos = RecvData.IndexOf(Cmd)

        If pos >= 0 Then
            Dim stat As String = RecvData.Substring(pos + Cmd.Length())
            Dim bitarray As List(Of Boolean) = New List(Of Boolean)
            Dim i As Integer
            Dim j As Integer
            Dim bitchar As String

            If dev = DevName.X Or dev = DevName.C Then
                For i = 7 To 0 Step -1
                    bitchar = stat.Substring(i, 1)
                    If bitchar = "1" Then
                        bitarray.Add(True)
                    Else
                        bitarray.Add(False)
                    End If
                Next
            ElseIf dev = DevName.Y Then
                For i = 7 To 2 Step -1
                    bitchar = stat.Substring(i, 1)
                    If bitchar = "1" Then
                        bitarray.Add(True)
                    Else
                        bitarray.Add(False)
                    End If
                Next
            ElseIf dev = DevName.T Then
                For j = 0 To 1
                    For i = 7 To 0 Step -1
                        bitchar = stat.Substring(i, 1)
                        If bitchar = "1" Then
                            bitarray.Add(True)
                        Else
                            bitarray.Add(False)
                        End If
                    Next
                    stat = stat.Substring(9)
                Next

            ElseIf dev = DevName.M Then
                For j = 0 To 7
                    For i = 7 To 0 Step -1
                        bitchar = stat.Substring(i, 1)
                        If bitchar = "1" Then
                            bitarray.Add(True)
                        Else
                            bitarray.Add(False)
                        End If
                    Next
                    stat = stat.Substring(9)
                Next
            End If
            Return bitarray
        Else
            Throw New Exception("正しく応答されませんでした。")
            Return Nothing
        End If
    End Function

    ''' <summary>
    ''' 引数で指定したコマンド文字列を送信し、正常に受け取ったかをBoolean型で返す
    ''' </summary>
    ''' <param name="cmd">送信コマンド文字列</param>
    ''' <returns>True: 正常に受け取った  False: 異常がありDIPPLCが??を返してきた</returns>

    Public Function SendCmd(ByRef cmd As String) As Boolean
        If Not (IsOpen) Then
            Throw New Exception("ポートが開かれていません。")
        End If

        Dim RecvData As String = ""

        MyBase.ReadExisting()

        MyBase.ReceivedBytesThreshold = MyBase.ReadBufferSize
        MyBase.WriteLine(cmd)

        Threading.Thread.Sleep(WaitTime)

        Do While (MyBase.BytesToRead > 0)
            Try
                '                RecvData &= MyBase.ReadLine() & vbCrLf
                RecvData &= MyBase.ReadExisting()
            Catch ex As TimeoutException
                Exit Do
            End Try
        Loop
        MyBase.ReceivedBytesThreshold = 1

        If RecvData.IndexOf(Me.CMD.ERR_CODE) >= 0 Then
            '        If RecvData.IndexOf("??") >= 0 Then
            Return False
        Else
            Return True
        End If
    End Function

    ''' <summary>
    ''' 引数で指定したコマンド文字列を送信し、応答データを戻り値にして返す。
    ''' </summary>
    ''' <param name="cmd">送信コマンド文字列</param>
    ''' <returns>DIPPLCから送られてきたデータ</returns>
    Public Function QueryCmd(ByRef cmd As String) As String
        If Not (IsOpen) Then
            Throw New Exception("ポートが開かれていません。")
        End If

        Dim RecvData As String = ""

        MyBase.ReadExisting()
        MyBase.ReceivedBytesThreshold = MyBase.ReadBufferSize
        MyBase.WriteLine(cmd)

        Threading.Thread.Sleep(WaitTime)

        Do While (MyBase.BytesToRead > 0)
            Try
                '                RecvData &= MyBase.ReadLine() & vbCrLf
                RecvData &= MyBase.ReadExisting()

            Catch ex As TimeoutException
                Exit Do
            End Try
        Loop
        MyBase.ReceivedBytesThreshold = 1

        Return RecvData
    End Function
    ''' <summary>
    ''' バージョンを要求する文字列の送信メソッド。応答の文字列を返す。
    ''' </summary>
    ''' <returns></returns>
    Public Function getVersion() As String
        If Not (IsOpen) Then
            Throw New Exception("ポートが開かれていません。")
        End If

        Dim cmd As String = Me.CMD.EXE_VIEW
        Dim recv As String
        Dim tempTime As Integer = Me.WaitTime
        WaitTime = 500
        recv = QueryCmd(cmd)
        WaitTime = tempTime
        Return recv
    End Function

    ''' <summary>
    ''' 指定した１つビットデバイスの状態を指定した状態に変更要求を出す。ラダーの実行状態により、変化してももとに戻ることがある。
    ''' </summary>
    ''' <param name="dev">DevName列挙型のY,Mを指定できる</param>
    ''' <param name="number">デバイスの番号</param>
    ''' <param name="stat">True:ＯＮ　False:ＯＦＦ</param>
    ''' <returns>True:正常終了　False:異常終了</returns>
    Public Function SetBitDevice(ByVal dev As DevName, ByVal number As Integer, ByVal stat As Boolean) As Boolean
        If Not (IsOpen) Then
            Throw New Exception("ポートが開かれていません。")
            Return False
        End If

        Dim cmd As String
        Dim DevStr As String
        Dim DevNum As String

        If (dev >= DevName.Y Or dev <= DevName.M) Then
            DevStr = DevNameStr(dev)
        Else
            Throw New Exception("使用できないデバイスが指定されました。")
            Return False
        End If
        Select Case dev
            Case DevName.Y
                If Not (number >= 0 And number <= 5) Then
                    Throw New Exception("使用できないデバイス番号が指定されました。")
                End If
            Case DevName.M
                If Not (number >= 0 And number <= 63) Then
                    Throw New Exception("使用できないデバイス番号が指定されました。")
                End If
            Case Else
                Throw New Exception("使用できないデバイスが指定されました。")
        End Select

        DevNum = "0" & Convert.ToString(number)
        DevNum = DevNum.Substring(DevNum.Length - 2)
        If stat = True Then
            cmd = DevStr & DevNum & "=" & "1"
        Else
            cmd = DevStr & DevNum & "=" & "0"
        End If

        Return SendCmd(cmd)
    End Function

    ''' <summary>
    ''' 指定した１つワードデバイスの値を指定した文字列の値に変更要求を出す。ラダーの実行状態により、変化してももとに戻ることがある。
    ''' </summary>
    ''' <param name="dev">DevName列挙型のDを指定できる</param>
    ''' <param name="number">デバイスの番号</param>
    ''' <param name="WordStr">数値を表す文字列</param>
    ''' <param name="syg">１０進数値か１６進数値かを指定</param>
    ''' <returns>True:正常終了　False:異常終了</returns>
    Public Function SetWordDevice(ByVal dev As DevName, ByVal number As Integer, ByVal WordStr As String, Optional ByVal syg As System.Globalization.NumberStyles = System.Globalization.NumberStyles.HexNumber) As Boolean
        If Not (IsOpen) Then
            Throw New Exception("ポートが開かれていません。")
            Return False
        End If

        Dim cmd As String
        Dim DevStr As String
        Dim DevNum As String
        Dim Data As Integer

        Select Case syg
            Case Globalization.NumberStyles.HexNumber

            Case Globalization.NumberStyles.Integer

            Case Else
                Throw New Exception("使用できないデータ型が指定されました。")
                Return False
        End Select

        If Int16.TryParse(WordStr, syg, Nothing, Data) = False Then
            Throw New Exception("変換できない文字列が指定されました。")
        End If

        If (dev = DevName.D) Then
            DevStr = DevNameStr(dev)
        Else
            Throw New Exception("使用できないデバイスが指定されました。")
            Return False
        End If
        If Not (number >= 0 And number <= 7) Then
            Throw New Exception("使用できないデバイス番号が指定されました。")
        End If

        DevNum = "0" & Convert.ToString(number)
        DevNum = DevNum.Substring(DevNum.Length - 2)
        If (syg = Globalization.NumberStyles.HexNumber) Then
            cmd = DevStr & DevNum & "=" & "x" & WordStr.ToUpper()
        Else
            cmd = DevStr & DevNum & "=" & Data.ToString()
        End If

        Return SendCmd(cmd)
    End Function

    ''' <summary>
    ''' 指定した１つワードデバイスの値を指定した数値に変更要求を出す。ラダーの実行状態により、変化してももとに戻ることがある。
    ''' </summary>
    ''' <param name="dev">DevName列挙型のDを指定できる</param>
    ''' <param name="number">デバイスの番号</param>
    ''' <param name="Data">新しい数値</param>
    ''' <returns>True:正常終了　False:異常終了</returns>
    Public Function SetWordDevice(ByVal dev As DevName, ByVal number As Integer, ByVal Data As Integer) As Boolean
        If Not (IsOpen) Then
            Throw New Exception("ポートが開かれていません。")
            Return False
        End If

        Dim cmd As String
        Dim DevStr As String
        Dim DevNum As String

        If (dev = DevName.D) Then
            DevStr = DevNameStr(dev)
        Else
            Throw New Exception("使用できないデバイスが指定されました。")
            Return False
        End If
        If Not (number >= 0 And number <= 7) Then
            Throw New Exception("使用できないデバイス番号が指定されました。")
        End If

        DevNum = "0" & Convert.ToString(number)
        DevNum = DevNum.Substring(DevNum.Length - 2)
        cmd = DevStr & DevNum & "=" & Data.ToString()

        Return SendCmd(cmd)
    End Function

    ''' <summary>
    ''' DIPPLCのラダーを指定ステップ番号以降を消去するコマンドを送信する
    ''' </summary>
    ''' <param name="number">開始ステップ番号</param>
    ''' <returns>True:正常終了　False:異常終了</returns>
    Public Function DelNimonic(ByVal number As Integer) As Boolean
        If Not (IsOpen) Then
            Throw New Exception("ポートが開かれていません。")
            Return False
        End If

        Dim cmd As String
        If number <= 0 Or number > 255 Then
            Throw New Exception("ステップ番号が間違っています。")
        End If

        cmd = Me.CMD.ERACE & number.ToString()

        Return SendCmd(cmd)
    End Function

    ''' <summary>
    ''' DIPPLCのラダーを全て消去するコマンドを送信する
    ''' </summary>
    ''' <returns>True:正常終了　False:異常終了</returns>
    Public Function DelAllNimonic() As Boolean
        If Not (IsOpen) Then
            Throw New Exception("ポートが開かれていません。")
            Return False
        End If

        Dim cmd As String

        cmd = Me.CMD.ERACE_ALL

        Return SendCmd(cmd)
    End Function

    ''' <summary>
    ''' DIPPLCのラダーを実行するコマンドを送信する
    ''' </summary>
    ''' <returns>True:正常終了　False:異常終了</returns>
    Public Function Run() As Boolean
        If Not (IsOpen) Then
            Throw New Exception("ポートが開かれていません。")
            Return False
        End If

        Dim cmd As String

        cmd = Me.CMD.EXE_CMD
        Return SendCmd(cmd)
    End Function

    ''' <summary>
    ''' DIPPLCのラダーを実行を停止するコマンドを送信する
    ''' </summary>
    ''' <returns>True:正常終了　False:異常終了</returns>
    Public Function RStop() As Boolean
        If Not (IsOpen) Then
            Throw New Exception("ポートが開かれていません。")
            Return False
        End If

        Dim RecvData As String = ""
        Dim sendData As Byte() = New Byte() {Me.CMD.EXE_PAUSE}
        MyBase.ReadExisting()

        Write(sendData, 0, 1)
        Threading.Thread.Sleep(WaitTime)

        Do
            Try
                RecvData &= MyBase.ReadLine() & vbCrLf
            Catch ex As TimeoutException
                Exit Do
            End Try
        Loop

        If RecvData.IndexOf(Me.CMD.ERR_CODE) >= 0 Then
            Return False
        Else
            Return True
        End If
    End Function

    ''' <summary>
    ''' YMCTを強制的にリセットします。
    ''' </summary>
    ''' <returns>True:正常終了　False:異常終了</returns>
    Public Function ResetBitDevice() As Boolean
        If Not (IsOpen) Then
            Throw New Exception("ポートが開かれていません。")
            Return False
        End If

        Dim cmd As String
        cmd = Me.CMD.EXE_RESET
        Return SendCmd(cmd)
    End Function
    ''' <summary>
    ''' X04ピンをＡＤ変換入力ピンに変更する
    ''' </summary>
    ''' <returns>True:正常終了　False:異常終了</returns>
    Public Function SetAD4() As Boolean
        Return Me.setADIO(ADIO_MODE.AD4)
    End Function

    ''' <summary>
    ''' X05ピンをＡＤ変換入力ピンに変更する
    ''' </summary>
    ''' <returns>True:正常終了　False:異常終了</returns>
    Public Function SetAD5() As Boolean
        Return Me.setADIO(ADIO_MODE.AD5)
    End Function

    ''' <summary>
    ''' X04ピンをデジタル変換入力ピンに変更する
    ''' </summary>
    ''' <returns>True:正常終了　False:異常終了</returns>
    Public Function SetDI4() As Boolean
        Return Me.setADIO(ADIO_MODE.DI4)
    End Function

    ''' <summary>
    ''' X05ピンをデジタル変換入力ピンに変更する
    ''' </summary>
    ''' <returns>True:正常終了　False:異常終了</returns>
    Public Function SetDI5() As Boolean
        Return Me.setADIO(ADIO_MODE.DI5)
    End Function

    ''' <summary>
    ''' X04ピンをデジタル変換入力ピンに変更する
    ''' </summary>
    ''' <returns>True:正常終了　False:異常終了</returns>
    Public Function setADIO(ByVal mode As ADIO_MODE) As Boolean
        Dim flag As Boolean
        Dim cmd As String

        Select Case mode
            Case ADIO_MODE.AD4
                cmd = Me.CMD.SET_AD4
            Case ADIO_MODE.AD5
                cmd = Me.CMD.SET_AD5
            Case ADIO_MODE.DI4
                cmd = Me.CMD.SET_DI4
            Case ADIO_MODE.DI5
                cmd = Me.CMD.SET_DI5
            Case Else
                Return False
        End Select

        Return SendCmd(cmd)
    End Function

    ''' <summary>
    ''' D0-D7までを強制的に０にします。
    ''' </summary>
    ''' <returns>True:正常終了　False:異常終了</returns>
    Public Function ResetWordDevice() As Boolean
        If Not (IsOpen) Then
            Throw New Exception("ポートが開かれていません。")
            Return False
        End If

        Dim i As Integer
        Dim Flag As Boolean

        For i = 0 To 7
            Flag = SetWordDevice(DipPlc.DevName.D, i, 0)
            If Flag = False Then
                Return False
            End If
        Next
        Return True
    End Function

    ''' <summary>
    ''' リモートストップして、ビットデバイス、ワードデバイスすべてをクリアします。
    ''' </summary>
    ''' <returns>True:すべて実行完了  False:失敗 </returns>
    Public Function Reset() As Boolean
        If Not (IsOpen) Then
            Throw New Exception("ポートが開かれていません。")
            Return False
        End If

        Dim Flag As Boolean
        Flag = RStop()
        If Flag = False Then
            Return False
        End If

        Flag = ResetBitDevice()

        If Flag = False Then
            Return False
        End If

        Flag = ResetWordDevice()
        If Flag = False Then
            Return False
        Else
            Return True
        End If
    End Function

    ''' <summary>
    ''' SerialPortのDataReceivedイベントから呼び出されるイベントプロシージャの定義
    ''' </summary>
    ''' <param name="sender">イベント発生対象となるオブジェクト</param>
    ''' <param name="e">イベントの詳細</param>
    Private Sub DipPlc_DataReceived(sender As Object, e As SerialDataReceivedEventArgs)
        If IsOpen = False Then
            Return
        End If

        Try
            mtx.WaitOne()
            recvStr &= ReadExisting()
            Trace.WriteLine(recvStr)
        Catch ex As Exception
            Trace.WriteLine(ex.ToString())
        Finally
            mtx.ReleaseMutex()
        End Try
    End Sub
    ''' <summary>
    ''' データ受信のイベントを許可するメソッド（これ以降,DipPlc_DataReceivedが呼び出される。）
    ''' </summary>
    Public Sub DataReceiveEventEnable()
        If IsOpen Then
            If handleFlag = False Then
                ReadExisting() ' 受信バッファのクリア
                AddHandler DataReceived, AddressOf DipPlc_DataReceived
                handleFlag = True
            Else
                MessageBox.Show("イベントの有効化は１回しか実行できません。")
            End If
        Else
            MessageBox.Show("ポートが開かれていません")
        End If
    End Sub

    ''' <summary>
    ''' データ受信のイベントを不許可にするメソッド（これ以降,DipPlc_DataReceivedが呼び出されない。）
    ''' </summary>
    Public Sub DataReceiveEventDisable()
        If IsOpen Then
            If handleFlag = True Then
                RemoveHandler DataReceived, AddressOf DipPlc_DataReceived
                handleFlag = False
            Else
                MessageBox.Show("イベントの無効化は有効時にしか実行できません。")
            End If
        Else
            MessageBox.Show("ポートが開かれていません")
        End If
    End Sub

    ''' <summary>
    ''' 非同期通信で受信したデータを全て取得して返す関数。取り出したあとデータは消去される。
    ''' </summary>
    ''' <returns>現在まで受信した文字列</returns>
    Public Function ReadRecvExisting() As String
        Dim tempstr As System.Text.StringBuilder
        Dim i As Integer

        If (recvStr <> Nothing) Then
            Try
                mtx.WaitOne()
                Dim Len As Integer = recvStr.Length()
                tempstr = New System.Text.StringBuilder(recvStr)
                recvStr = ""
            Catch ex As Exception
                Trace.WriteLine(ex.ToString())
            Finally
                mtx.ReleaseMutex()
            End Try

            Try
                If (tempstr.Length >= 0) Then
                    For i = 0 To tempstr.Length - 1
                        If tempstr(i) = ChrW(5) Then
                            tempstr(i) = " "c
                        End If
                    Next
                Else
                    tempstr.Append("")
                End If
            Catch ex As Exception
                tempstr = New System.Text.StringBuilder("")
            End Try

        Else
            tempstr = New System.Text.StringBuilder("")
        End If
        Return tempstr.ToString()
    End Function

    ''' <summary>
    ''' 非同期通信で受信したデータを１行分取得して返す関数。取得した文字列は削除される。
    ''' </summary>
    ''' <returns>現在まで受信した文字列の１行分</returns>
    Public Function ReadRecvLine() As String
        Dim tempstr As System.Text.StringBuilder


        If recvStr <> Nothing Then
            Try
                mtx.WaitOne()
                Dim pos As Integer = recvStr.IndexOf(MyBase.NewLine)
                If pos >= 0 Then
                    tempstr = New System.Text.StringBuilder(recvStr.Substring(0, pos))
                    recvStr = recvStr.Substring(pos + MyBase.NewLine.Length)
                Else
                    tempstr = New System.Text.StringBuilder("")
                End If
            Catch ex As Exception
                Trace.WriteLine(ex.ToString())
            Finally
                mtx.ReleaseMutex()
            End Try

            Try
                For i = 0 To tempstr.Length - 1
                    If tempstr(i) = ChrW(5) Then
                        tempstr(i) = " "c
                    End If
                Next
            Catch ex As Exception
                tempstr = New System.Text.StringBuilder("")
            End Try
        Else
            tempstr = New System.Text.StringBuilder("")
        End If
        Return tempstr.ToString()
    End Function
End Class
